#include "windows.h"
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>

#include "ps2000.h"

#include "util.h"

#define CAPTURE_LENGTH 1024

#define MAX(a,b) ((a > b) ? a : b)
#define MIN(a,b) ((a < b) ? a : b)

int main(int argc, char *argv[]) {
	short handle, timebase, oversample = 1, time_units, retval, overflow;
	long time_interval, max_samples, time_indisposed_ms;
	short values[CAPTURE_LENGTH];
	short levels[5];
	int i, j, error;
	long avg, min, max;
	char temp[80];
	
	if(argc < 2) {
		printf("Usage: calibrate <levels_file>\n");
		return -1;
	}
		
	handle = ps2000_open_unit();
	
	ps2000_set_channel(handle, PS2000_CHANNEL_A, TRUE, 1, PS2000_500MV);
	ps2000_set_channel(handle, PS2000_CHANNEL_B, FALSE, 1, PS2000_500MV);
	
	ps2000_set_trigger(handle, PS2000_NONE, 0, PS2000_RISING, 0, 0);
	
	for(timebase=0; timebase<PS2000_MAX_TIMEBASE; timebase++) {
		retval = ps2000_get_timebase(handle, timebase, CAPTURE_LENGTH, 
			&time_interval, &time_units, oversample, &max_samples);
		printf("[%d = %d] Time interval: %ld, time units: %d, max samples: %ld\n", 
			timebase, retval, time_interval, time_units, max_samples);
	}
	
	printf("Select timebase: ");
	fflush( stdin );
	scanf ( "%hi", &timebase );
	
	ps2000_get_timebase(handle, timebase, CAPTURE_LENGTH, 
		&time_interval, &time_units, oversample, &max_samples);

	fflush( stdin );
		
	for(j=0, error=0; j<5 && !error; j++) {
		if(j)
			printf("Connect pin #%d to VCC and others to GND and press ENTER...\n", j);
		else
			printf("Connect all pins to GND and press ENTER...\n");
			
		fgets(temp, sizeof(temp), stdin);
		ps2000_run_block(handle, CAPTURE_LENGTH, timebase, oversample, &time_indisposed_ms);
	
		while (!ps2000_ready(handle))
			Sleep(100);
	
		ps2000_get_values(handle, values, NULL, NULL, NULL, &overflow, CAPTURE_LENGTH);
		
		max = -1000000; min = 1000000; avg = 0;
		
		for(i=0; i<CAPTURE_LENGTH; i++) {
			max = MAX(max, values[i]);
			min = MIN(min, values[i]);
			avg += values[i];
		}
		
		avg /= CAPTURE_LENGTH;
		levels[j] = avg;
		
		printf("%ld - %ld, average %hi\n", min, max, levels[j]);				
		
		if(j >= 1 && levels[j] < levels[j-1]) {
			printf("Error: Every voltage reading should be higher than the previous ones!\n");
			error = 1;
		}
	}
		
	ps2000_stop(handle);

	ps2000_close_unit(handle);
	
	if(!error)
		save_levels(argv[1], levels);
	
	return 0;
}
