#include "windows.h"
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>

#include "ps2000.h"

#include "util.h"

#define CAPTURE_LENGTH 10000000

short * streaming_buffer;
long streaming_ptr;

void  __stdcall ps2000FastStreamingReady(short **overviewBuffers, 
		short overflow, unsigned long triggeredAt, short triggered, 
		short auto_stop, unsigned long nValues) {
	long i;
	short * channel_A = overviewBuffers[0];
	
	for(i=0; i<nValues && streaming_ptr < CAPTURE_LENGTH; i++)
		streaming_buffer[streaming_ptr++] = channel_A[i];
}

int main(int argc, char *argv[]) {
	short handle;
	int i, coverage, filter_level, capture_interval;
	char level_conv[65536];
	
	if(argc < 5) {
		printf("Usage: stream <levels_file> <coverage-%%> <filter-level> <capture_interval_ns>\n");
		printf("Example: stream levels.txt 80 2\n");
		return -1;
	}
	
	coverage = atoi(argv[2]);
	filter_level = atoi(argv[3]);
	capture_interval = atoi(argv[4]);
	
	load_levels(argv[1], level_conv, coverage);
		
	handle = ps2000_open_unit();
	
	ps2000_set_channel(handle, PS2000_CHANNEL_A, TRUE, 1, PS2000_500MV);
	ps2000_set_channel(handle, PS2000_CHANNEL_B, TRUE, 1, PS2000_500MV);
	
	// No trigger
	ps2000_set_trigger(handle, PS2000_NONE, 0, PS2000_RISING, 0, 0);
	
	// Trigger on rising voltage on channel B
	//ps2000_set_trigger(handle, PS2000_CHANNEL_B, 8000, PS2000_RISING, -1, 10000);
	
	streaming_buffer = (short *)malloc(sizeof(short) * CAPTURE_LENGTH);
	streaming_ptr = 0;
	
	// Collect a maximum of 100 000 samples, no auto-stop, no aggregation (1 samples aggregated), 50 k overview buffer
	if(capture_interval < 1000)
		ps2000_run_streaming_ns(handle, capture_interval, PS2000_NS, 1000000, 0, 1, 500000);
	else
		ps2000_run_streaming_ns(handle, capture_interval/1000, PS2000_US, 1000000, 0, 1, 500000);
	
	while(!_kbhit() && streaming_ptr < CAPTURE_LENGTH) {
		ps2000_get_streaming_last_values(handle, ps2000FastStreamingReady);
		//printf("%ld\n", streaming_ptr);
		Sleep (0);
	}
	
	ps2000_stop(handle);

	ps2000_close_unit(handle);

	for(i=0; i<CAPTURE_LENGTH; i++)
		streaming_buffer[i] = level_conv[streaming_buffer[i]+32768];
		
	clean_signal(streaming_buffer, CAPTURE_LENGTH, filter_level);

	save_ols("output.ols", streaming_buffer, CAPTURE_LENGTH, 1000000000/capture_interval);
	//save_sump("output1.sla", streaming_buffer, CAPTURE_LENGTH, 1000000000/capture_interval);
	
	return 0;
}
