#include <stdio.h>
#include <stdlib.h>

#include "util.h"

void save_levels(char *filename, short *levels) {
	FILE *out = fopen(filename, "wt");
	int i, j;
	long sum;
	
	if(out == NULL)
		return; // fail silently
		
	for(i=1; i<=4; i++)
		levels[i] -= levels[0]; // increase relative to ground
	
	for(i=0; i<16; i++) {
		// calculate supposedly correct level
		sum = levels[0];
		for(j=0; j<4; j++)
			if(i & (1<<j))
				sum += levels[j+1];
				
		fprintf(out, "%ld\n", sum);
	}
	
	fclose(out);
}

void load_levels(char *filename, char *level_conv, int coverage) {
	int i, j, a, b, d;
	short levels[16];
	FILE *in;
	
	in = fopen(filename, "rt");
	
	for(i=0; i<16; i++)
		fscanf(in, "%hi", &levels[i]);	
		
	fclose(in);
	
	for(i=0; i<(int)levels[0]+32768; i++)
		level_conv[i] = 0;
		
	for(j=1; j<16; j++) {
		a = levels[j-1];
		b = levels[j];
		d = b-a;
		
		for(i = a + 32768; i < a + d * coverage / 200 + 32768; i++)
			level_conv[i] = j-1;
			
		for(; i < b - d * coverage / 200 + 32768; i++)
			level_conv[i] = -1;
			
		for(; i < b + 32768; i++)
			level_conv[i] = j;
	}
	
	for(i=levels[15]+32768; i<65536; i++)
		level_conv[i] = 15;
		
}

void clean_signal(short *values, int len, int filter_level) {
	int i, j, last_valid;
	
	for(i=0; i<len && values[i] == -1; i++) {} // go to first valid value
	
	if(i==len)
		return; // damn, silently fail...
		
	for(j=0; j<i; j++)
		values[j] = values[i];
		
	last_valid = values[i];
		
	for(i++; i<len; i++) { // remove invalids
		if(values[i] == -1)
			values[i] = last_valid;
		else
			last_valid = values[i];
	}
	
	j = 0; // start of current value
	
	do {
		//printf("j=%d  i=%d\n", i, j);
		
		for(i=j+1; i<len && values[i] == values[j]; i++) {}
		
		if(i == len)
			break;
			
		//printf("  j=%d  i=%d\n", i, j);
		if(i-j <= filter_level) // fill out too short sequences
			for(; i>j; i--)
				values[i-1] = values[i];
		else // otherwise start a new block
			j = i;
	} while(i<len);

	// if the end is too short, fill it with previous value
	if(i-j <= filter_level)
		for(; j<len; j++)
			values[j] = values[j-1];
}

void save_sump(char *filename, short *values, int len, int rate) {
	int i;
	FILE *out = fopen(filename, "wt");
	
	fprintf(out, ";Size: %d\n", len);
	fprintf(out, ";Rate: %d\n", rate);
	fprintf(out, ";Channels: 32\n");
	fprintf(out, ";EnabledChannels: 255\n");
	
	for(i=0; i<len; i++)
		fprintf(out, "%08X\n", values[i]);
		
	fclose(out);
}

void save_ols(char *filename, short *values, int len, int rate) {
	int i, j, prev = -1;
	FILE *out = fopen(filename, "wt");
	
	fprintf(out, ";Rate: %d\n", rate);
	fprintf(out, ";Channels: 8\n");
	
	for(i=j=0; i<len; i++)
		if(prev != values[i]) {
			fprintf(out, "%02X@%d\n", values[i], j++);
			prev = values[i];
		}
		
	fclose(out);
}
